import logging
import os
import sys
from io import TextIOWrapper

import click
import coloredlogs

import shortlinks
from shortlinks.generator import ShortlinkGenerator

if sys.stdout.isatty():
    # You're running in a real terminal
    LOG_FORMAT = "%(asctime)s %(name)s %(levelname)s %(message)s"
else:
    LOG_FORMAT = "%(name)s %(levelname)s %(message)s"

LOG_FIELD_STYLES = {
    "asctime": {"color": "green"},
    "hostname": {"color": "magenta"},
    "levelname": {"bold": True, "bright": True, "color": "black"},
    "name": {"color": "blue"},
    "programname": {"color": "cyan"},
    "username": {"color": "yellow"},
}

coloredlogs.install(level="DEBUG", fmt=LOG_FORMAT, field_styles=LOG_FIELD_STYLES)
logger = logging.getLogger("shortlinks")


@click.group()
def cli():
    logger.setLevel(getattr(logging, os.getenv("LOGLEVEL", "INFO").upper()))


@cli.command()
def version(verbose):
    """Displays the version"""
    click.echo("Version: %s" % shortlinks.__version__)


@cli.command()
@click.argument("redirects_file", type=click.File("r"), envvar="REDIRECTS_FILE")
@click.option(
    "-o",
    "--out-dir",
    type=click.Path(),
    default="public",
    envvar="OUT_DIR",
    help="Output directory",
)
@click.option(
    "ignore_urls",
    "-i",
    "--ignore-url",
    multiple=True,
    default=[],
    envvar="IGNORE_URLS",
    help="Ignore HTTP errors for this URL, can be specified multiple times",
)
@click.option(
    "-c", "--check", is_flag=True, envvar="CHECK_MODE", help="Run in check mode"
)
def generate(redirects_file: TextIOWrapper, out_dir, ignore_urls, check):
    """Generate shortlink filetree"""
    generator = ShortlinkGenerator(redirects_file, ignore_urls)
    if check:
        if not generator.check_all_urls():
            sys.exit(1)
    else:
        generator.generate(out_dir)


if __name__ == "__main__":
    cli()
